#include "util.h"

#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

const char* ADC_name[] = {"H1", "H2", "T", "L"};

#define AREF 5
#define ADC_INIT_CHANNEL 0

const int num_channels = sizeof(ADC_name)/sizeof(ADC_name[0]);
int ADC_value[num_channels];

/* intreruperea de comparatie cu OCR1A */
ISR(TIMER1_COMPA_vect)
{
    // TODO 4 
    // start an ADC conversion
    ADCSRA |= (1 << ADSC);
}

/* intreruperea pentru pin change PB0 si PB1 */
ISR(PCINT1_vect)
{
    // PB0 apasat
    if((PINB & (1 << PB0)) == 0)
    {
        PORTB &= ~(1 << PB3);
        OCR1A -= 1000;
    }
    
    // PB1 apasat
    if((PINB & (1 << PB1)) == 0)
    {
        PORTB |= (1 << PB3);
        OCR1A += 1000;
    }
}

ISR(ADC_vect)
{
    // TODO 4
    // * store new result
    // * change channel
    // * start a new conversion if there are any channels left
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    channel = (channel + 1) %  (num_channels);
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

void ADC_init(void)
{
    ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
    ADCSRA = (1 << ADEN) | (1 << ADIE) | (5 << ADPS0);
}

/*
    Returns the result of an ADC conversion. 
    Starts a conversion and waits for the result when not using interrupts.
*/
int ADC_get(uint8_t channel)
{
    return ADC_value[channel];
}

/*
    Converts a value given by the ADC to volts.
*/
double ADC_voltage(int raw)
{
    (void)raw;
    // TODO 2
    return raw * AREF / 1024.;
}

/*
    Prints on the T6963 LCD the value of each ADC channel.
*/
void ADC_show(void)
{
    // TODO 1
    // print ADC channels 0-5 starting at line 0, column 0
    // use ADC_name[] for the name of the channel
    /*int i;
    for(i = 0; i < 6; ++i)
    {
        T6963cPutStringXY(0, i, ADC_name[i]);

        char value[10];
        sprintf(value, "%.2fV", ADC_voltage(ADC_get(i)));

        T6963cPutStringXY(6, i, "    ");
        T6963cPutStringXY(6, i, value);
    }*/
	char value[10];
    LCD_writeInstr(LCD_INSTR_clearDisplay);

    sprintf(value, "H1:%.2f", ADC_voltage(ADC_get(0)));
    LCD_printAt(0, value);

    sprintf(value, "H2:%.2f", ADC_voltage(ADC_get(1)));
    LCD_printAt(8, value);

    sprintf(value, "T :%.2f", ADC_voltage(ADC_get(2)));
    LCD_printAt(40, value);

    sprintf(value, "L :%.2f", ADC_voltage(ADC_get(3)));
    LCD_printAt(72, value);
}

/* initializeaza intrarile si iesirile */
void IO_init()
{
    // LED-ul RGB este pus ca iesire
    DDRD |= (1 << PD4) | (1 << PD7);
    DDRB |= (1 << PB3);
    
    // butoanele sunt puse ca intrare, cu rezistenta de pull-up
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= ((1 << PB0) | (1 << PB1));
    
    // intreruperile sunt activate pentru PCINT8 (PB0) si PCINT9 (PB1)
    PCICR  = (1 << PCIE1);
    PCMSK1 = (1 << PCINT8) | (1 << PCINT9);
}

/* initializeaza timer-ul 1 pentru exercitiile 1-4 */
void timer1_init()
{
    // pragul initial de comparare
    OCR1A = 15625;
    
    // activeaza intreruperea de comparare cu OCR1A
    TIMSK1 = (1 << OCIE1A);
    
    // modul CTC cu OCR1A
    // prescaler 1024
    TCCR1B = (5 << CS10) | (1 << WGM12);
}

int main(void)
{
    // initializari si testare interfeta afisare/debug
    UI_init();
    printf_init();
    //printf("USART ok\n");
	
	//button
    DDRD &= ~(1 << PD6);
    PORTD |= (1 << PD6);
    int8_t PD6_flag = 0;

    //led
    DDRD |= (1 << PD7);
	
	ADC_init();

    // codul relevant laboratorului incepe aici
    sei();

    timer1_init();
    //IO_init();

	char s[10];
    sprintf(s, "AT\r\n");
    char buff[1024];
	
    while(1)
    {
        //drawProgressBars(TCNT1, OCR1A, 0xFFFF); // pentru afisare, ignorati
        //_delay_ms(10);
		if(!(PIND & (1 << PD6)) && !PD6_flag)
        {
            PD6_flag = 1;
            ADC_show();
            //PORTD ^= (1 << PD7);
        }

        if((PIND & (1 << PD6)) && PD6_flag)
        {
            PD6_flag = 0;

            USART0_print(s);
            _delay_ms(9);
            LCD_writeInstr(LCD_INSTR_clearDisplay);
            USART0_scan(buff);
            LCD_printAt(0,buff);
            USART0_scan(buff);
            LCD_printAt(8,buff);

        }
        _delay_ms(5);
    }
    
    return 0;
}

